/** Copyright (C) 2017 by Bundesanstalt für Gewässerkunde
 * Software engineering by
 *  Björnsen Beratende Ingenieure GmbH
 *  Dr. Schumacher Ingenieurbüro für Wasser und Umwelt
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.artifacts.sinfo.predefinedcollision;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.dive4elements.artifactdatabase.state.DefaultOutput;
import org.dive4elements.artifactdatabase.state.Facet;
import org.dive4elements.artifactdatabase.state.FacetActivity;
import org.dive4elements.artifacts.Artifact;
import org.dive4elements.artifacts.ArtifactFactory;
import org.dive4elements.artifacts.CallMeta;
import org.dive4elements.artifacts.common.utils.XMLUtils;
import org.dive4elements.river.artifacts.AbstractStaticStateArtifact;
import org.dive4elements.river.artifacts.D4EArtifact;
import org.dive4elements.river.artifacts.model.FacetTypes;
import org.dive4elements.river.artifacts.states.StaticState;
import org.w3c.dom.Document;

/**
 * Display of a river bed collision data series loaded from database
 *
 * @author Matthias Schäfer
 *
 */
public class PredefinedCollisionArtifact extends AbstractStaticStateArtifact implements FacetTypes {

    private static final long serialVersionUID = 1L;

    /** The log for this class. */
    private static Logger log = Logger.getLogger(PredefinedCollisionArtifact.class);

    /** Artifact name. */
    private static final String NAME = "collision";

    static {
        // TODO: Move to configuration.
        FacetActivity.Registry.getInstance().register(NAME, FacetActivity.INACTIVE);
    }

    public static final String STATIC_STATE_NAME = "state.predefined_collision.static";

    /**
     * Trivial Constructor.
     */
    public PredefinedCollisionArtifact() {
        log.debug("new PredefinedCollisionArtifact");
    }

    @Override
    public String getName() {
        return NAME;
    }

    /**
     * Gets called from factory, to set things up.
     */
    @Override
    public void setup(final String identifier, final ArtifactFactory factory, final Object context, final CallMeta callMeta, final Document data,
            final List<Class> loadFacets) {

        log.debug("PredefinedCollisionArtifact.setup");

        if (log.isDebugEnabled()) {
            log.debug(XMLUtils.toString(data));
        }

        final String code = getDatacageIDValue(data);
        final String[] parts = code.split("-");
        if (parts.length >= 2)
            addStringData("collision_id", parts[1]);
        if (parts.length >= 3)
            addStringData("year", parts[2]);
        else
            addStringData("year", "jahr?");
        String seriesName;
        if (parts.length >= 4)
            seriesName = parts[3];
        else
            seriesName = "name?";

        // if different facet wanted, e.g. for each year use different facetIndex
        // if multiple Result treated, get each Result from data
        createFacets(callMeta, code, seriesName, getDataAsString("year"));

        super.setup(identifier, factory, context, callMeta, data, loadFacets);
    }

    private void createFacets(final CallMeta callMeta, final String code, final String seriesName, final String yearForName) {
        if (code == null)
            return;

        final String[] parts = code.split("-");
        if (parts.length < 2) {
            log.error("Invalid datacage ID '" + code + "'");
            return;
        }
        final ArrayList<Facet> facets = new ArrayList<>(1);
        facets.add(PredefinedCollisionProcessor.createFacet(0, callMeta, yearForName));
        // facets.add(CollisionGaugeWProcessor.createFacet(callMeta, seriesName)); //REMARK gauge_w is the cm above gauge datum,
        // therefore only useful with transformation to NHN
        addFacets(STATIC_STATE_NAME, facets);
    }

    @Override
    protected void initStaticState() {

        log.debug("initStaticState " + getName() + " " + identifier());

        final StaticState state = new StaticState(STATIC_STATE_NAME);
        final DefaultOutput output = new DefaultOutput("general", "general", "image/png", "chart");

        final List<Facet> facets = getFacets(STATIC_STATE_NAME);
        output.addFacets(facets);
        state.addOutput(output);

        setStaticState(state);
    }

    @Override
    protected void initialize(final Artifact artifact, final Object context, final CallMeta meta) {
        // do not clone facets, etc. from master artifact

        log.debug("initialize");
        importData((D4EArtifact) artifact, "river");
        importData((D4EArtifact) artifact, "ld_from");
        importData((D4EArtifact) artifact, "ld_to");

        log.debug("ld_from " + getDataAsString("ld_from"));
        log.debug("ld_to " + getDataAsString("ld_to"));
    }
}